#!/var/lib/ecp-veil/controller/env/bin/python3

import argparse
import os
import subprocess
import signal

from veilrpc.utils.bazalt import BASE_SCRIPT_PATH, RUN_PATH, check_pid
UPGRADE_SCRIPT_PATH = f"{BASE_SCRIPT_PATH}/veil-upgrade"
UPGRADE_PID_FILE = RUN_PATH + "/veil-upgrade.pid"
UPGRADE_LOG_FILE = RUN_PATH + "/veil-upgrade.log"


def check_already_running():
    if os.path.isfile(UPGRADE_PID_FILE):
        with open(UPGRADE_PID_FILE, 'r') as f:
            pid_str = f.readline().rstrip()
            if pid_str:
                return check_pid(int(pid_str))


def start(only):
    pid = os.fork()
    if pid == 0:
        # child process
        os.setpgid(os.getpid(), os.getpid())
        with open(UPGRADE_LOG_FILE, 'w') as log_file:
            cmd = [UPGRADE_SCRIPT_PATH, '-p']
            if only:
                cmd.append('-u')
            process = subprocess.Popen(cmd,
                                       stdout=log_file,
                                       stderr=subprocess.STDOUT,
                                       stdin=subprocess.DEVNULL)
        with open(UPGRADE_PID_FILE, 'w') as pid_file:
            pid_file.write(str(process.pid))
        os.kill(os.getpid(), signal.SIGTERM)
    else:
        # parent process
        os.waitpid(pid, 0)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-u', '--only', action='store_true',
                        help="Upgrade only controller.")

    options = parser.parse_args()
    if check_already_running():
        print('Upgrade already running. Use \"upgrade log\" command to see progress\n')
        return
    start(options.only)


if __name__ == "__main__":
    main()
