"""
(c) 2024 LLC «DACOM M»

Django settings for bazalt project.
`
Generated by 'django-admin startproject' using Django 1.10.

For more information on this file, see
https://docs.djangoproject.com/en/1.10/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.10/ref/settings/
"""

from cryptography.hazmat.primitives.serialization import load_pem_private_key, load_pem_public_key
import datetime
from multidict import istr
import os

from veilrpc.utils.bazalt import Sentry
Sentry('controller')

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.10/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '%i3&w*7qmddl-fm^7wvt$sz0@us_^65tm=zw^vc)1jq*()qj!q'  # nosec

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []

# Application definition

INSTALLED_APPS = [
    'daphne',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.staticfiles',
    'rest_framework',
    'dry_rest_permissions',
    'authentication_directory',
    'base',
    'task',
    'node',
    'event',
    'shared_storage',
    'cluster_storage',
    'datapool',
    'vdisk',
    'domain',
    'cluster',
    'iso',
    'license',
    'translation',
    'ssl_certificate',
    'user',
    'user_group',
    'permission',
    'snapshot',
    'hint',
    'library',
    'datacenter',
    'controller',
    'tag',
    'storage_transport',
    'zfs_pool',
    'networking',
    'resource_pool',
    'networking.kernel',
    'networking.control',
    'networking.data',
    'networking.extnetworks',
    'networking.dpi',
    'iscsi_target',
    'drf_spectacular',
    'drf_spectacular_sidecar',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'bazalt.middleware.token_middleware.TokenValidationMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware'

]

ROOT_URLCONF = 'bazalt.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'swagger_to_pdf/templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

STATICFILES_DIRS = [
    os.path.join(BASE_DIR, "swagger_to_pdf/static"),
]

WSGI_APPLICATION = 'bazalt.wsgi.application'
ASGI_APPLICATION = "bazalt.websockets.asgi.application"

# Database
# https://docs.djangoproject.com/en/1.10/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql',
        'NAME': 'bazalt',
        'USER': 'bazalt',
        'PASSWORD': 'hizVo5l1Z5fkMSfCsp9J',
        'HOST': '127.0.0.1',
        'PORT': '5432',
        'ATOMIC_REQUESTS': True,
        'OPTIONS': {
            'sslmode': 'disable',
        },
    }
}

PASSWORD_HASHERS = (
    'utils.password.StreebogHasher',
    'django.contrib.auth.hashers.PBKDF2PasswordHasher',
    # 'django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher',
    # 'django.contrib.auth.hashers.BCryptPasswordHasher',
    # 'django.contrib.auth.hashers.SHA1PasswordHasher',
    # 'django.contrib.auth.hashers.MD5PasswordHasher',
    # 'django.contrib.auth.hashers.CryptPasswordHasher',
)

REST_FRAMEWORK = {
    'DEFAULT_PERMISSION_CLASSES': (
        'rest_framework.permissions.IsAuthenticated',
    ),
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework_jwt.authentication.JSONWebTokenAuthentication',
    ),
    'DEFAULT_FILTER_BACKENDS': (
        'django_filters.rest_framework.DjangoFilterBackend',
    ),
    'DEFAULT_PAGINATION_CLASS': 'bazalt.rest.pagination.CustomPagination',
    'PAGE_SIZE': 100,
    'EXCEPTION_HANDLER': 'bazalt.rest.generics.custom_exception_handler',
    'TEST_REQUEST_DEFAULT_FORMAT': 'json',
    'DEFAULT_SCHEMA_CLASS': 'drf_spectacular.openapi.AutoSchema',
}

SPECTACULAR_SETTINGS = {
    'TITLE': 'Controller API',
    'DESCRIPTION': 'Controller description',
    'VERSION': '6.x.x',
    'SERVE_INCLUDE_SCHEMA': False,
    'CONTACT': {"email": "support@spacevm.ru"},
    'TOS': "https://spacevm.ru",
    'DISABLE_ERRORS_AND_WARNINGS': False
}

SESSION_EXPIRE_AT_BROWSER_CLOSE = True
SESSION_COOKIE_AGE = 1209600  # 2 недели
SESSION_CONCURRENT_COUNT = 100
SESSION_IDLE_TIMEOUT = 600  # 10 мин

JWT_EXPIRATION_DELTA_DAYS = 1
JWT_REFRESH_EXPIRATION_DELTA_DAYS = 4 * 7  # 4 недели

LOCALE_PATHS = (
    os.path.join(BASE_DIR, "locale"),
)

# Internationalization
# https://docs.djangoproject.com/en/1.10/topics/i18n/

LANGUAGES = [
    ('ru', 'Russian'),
    ('en', 'English'),
]

LANGUAGE_CODE = 'en'

# Путь к папке логов
LOGS_DIR = "/var/log/veil/controller"
BACKUP_DIR = f"{LOGS_DIR}/journal"

# Redis settings
REDIS_URL = 'redis://localhost:6379/0'
REDIS_TIMEOUT = 5

CONTROLLER_USER = 'veil-controller'

# websockify settings
WS_TOKEN_EXPTIME = 30

# websockets settings
WEBSOCKETS_URL = 'websockets'
WEBSOCKETS_HEARTBIT = False

# controller settings

# в /var/lib/ecp-veil/.users/veil-controller/.ssh/ лежит id_rsa,
# который используется для взаимодействия controller - node
# этот ключ добавлен в authorized_keys для пользователей root и veil-node
# на ноде. Для взаимодействия node-node ключ не используется.
BASE_PATH = '/var/lib/ecp-veil'
CTRL_SSH_PRIVATE_KEY = f'{BASE_PATH}/.users/veil-controller/.ssh/id_rsa'
CTRL_SSH_PUBLIC_KEY = f'{BASE_PATH}/.users/veil-controller/.ssh/id_rsa.pub'
CTRL_SSL_CERTS_PATH = f'{BASE_PATH}/certs'
CTRL_PATH = f'{BASE_PATH}/controller'
CTRL_KEYTABS_PATH = f'{CTRL_PATH}/krb_keytabs'
CTRL_KEYTAB_MAX_SIZE = 1  # Mbyte
DATA_PATH = f'{BASE_PATH}/data'
CTRL_YAML_PATH = f'{DATA_PATH}/modules'  # Одинаковая на всех узлах через lsync при установке или удалении узлов
CTRL_LOCAL_YAML_PATH = f'{DATA_PATH}/local_modules'
NGINX_YAML_CONF_PATH = f'{CTRL_LOCAL_YAML_PATH}/nginx.yaml'
NTP_FILE = f'{CTRL_LOCAL_YAML_PATH}/ntp.yaml'
SNMP_FILE = f'{CTRL_YAML_PATH}/snmp.yaml'
MIT_KRB5_FILE = f'{CTRL_YAML_PATH}/mit_krb5.yaml'
CTRL_LOCAL_DATA_PATH = f'{CTRL_PATH}/local_data/'
CTRL_DOMAINS_PATH = f'{CTRL_PATH}/domains/'

HOSTS_FILE_PATH = '/etc/hosts'
HOSTS_D_PATH = f'{HOSTS_FILE_PATH}.d/'
HOSTS_PATTERN = '/etc/hosts.d/{}.conf'
NODE_HOSTS_FILE_PATTERN = '{node_ip}\t\t{node_uuid}'
HOSTS_CMD = "sudo /usr/local/sbin/veil-hosts"

# worker user settings
NODE_PATH = f'{BASE_PATH}/node'
NODE_LOCAL_DATA_PATH = f'{NODE_PATH}/local_data/'
WORKER_USER = 'veil-node'
WORKER_ROOT_SSH_DIR = '/root/.ssh/'
WORKER_ROOT_AUTHORIZED_KEYS_PATH = f"{WORKER_ROOT_SSH_DIR}authorized_keys"
WORKER_USER_SSH_DIR = f"{BASE_PATH}/.users/{WORKER_USER}/.ssh/"
WORKER_USER_AUTHORIZED_KEYS_PATH = f"{WORKER_USER_SSH_DIR}authorized_keys"

GLUSTER_UNIX = True

OEM = "SpaceVM"
OEM_FULL = "SpaceVM"
OEM_SITE = "https://spacevm.ru/"

# sync proxy
SYNC_WAIT_TIMEOUT = 60
FORBIDDEN_SYNC_HEADER = istr('FORBIDDEN_SYNC')
DJANGO_URL = 'http://127.0.0.1:8080'

SILK = False

# ISOLATION_LEVEL_AUTOCOMMIT = 0
# ISOLATION_LEVEL_READ_UNCOMMITTED = 4
# ISOLATION_LEVEL_READ_COMMITTED = 1  DEFAULT!!!
# ISOLATION_LEVEL_REPEATABLE_READ = 2
# ISOLATION_LEVEL_SERIALIZABLE = 3
# ISOLATION_LEVEL_DEFAULT = None
ISOLATION_LEVEL = 1

HINTS_DEBUG = False
WEBSOCKETS_DEBUG = False

SYS_CLASS_NET_PARSER = True

try:
    from bazalt.settings_local import REDIS_URL
except ImportError:
    pass
CHANNEL_LAYERS = {
    'default': {
        'BACKEND': 'channels_redis.pubsub.RedisPubSubChannelLayer',
        'CONFIG': {
            "hosts": [REDIS_URL],
        },
    },
}

DATABASES['default']['OPTIONS']['isolation_level'] = ISOLATION_LEVEL

if SILK:
    INSTALLED_APPS.append('silk')
    MIDDLEWARE.append('silk.middleware.SilkyMiddleware')
    SILKY_PYTHON_PROFILER = True
    SILKY_PYTHON_PROFILER_BINARY = False
    SILKY_MAX_RECORDED_REQUESTS = 10 ** 4
    SILKY_MAX_RECORDED_REQUESTS_CHECK_PERCENT = 10
    SILKY_META = True

try:
    with open(os.path.join(BASE_DIR, 'bazalt/jwtRS256.key'), 'rb') as f:
        private = load_pem_private_key(f.read(), password=None)
    with open(os.path.join(BASE_DIR, 'bazalt/jwtRS256.key.pub'), 'rb') as f:
        public = load_pem_public_key(f.read())
    JWT_ALGORITHM = "RS256"
except Exception:
    private, public = None, None
    JWT_ALGORITHM = "HS256"


JWT_AUTH = {
    'JWT_GET_USER_SECRET_KEY': 'bazalt.rest.auth.bazalt_jwt_get_user_secret_key',
    'JWT_EXPIRATION_DELTA': datetime.timedelta(days=JWT_EXPIRATION_DELTA_DAYS),
    'JWT_PAYLOAD_HANDLER': 'bazalt.rest.auth.bazalt_jwt_payload_handler',
    'JWT_DECODE_HANDLER': 'bazalt.rest.auth.bazalt_jwt_decode_handler',
    'JWT_RESPONSE_PAYLOAD_HANDLER': 'bazalt.rest.auth.bazalt_jwt_response_payload_handler',
    'JWT_ALLOW_REFRESH': True,
    'JWT_REFRESH_EXPIRATION_DELTA': datetime.timedelta(days=JWT_REFRESH_EXPIRATION_DELTA_DAYS),
    'JWT_PUBLIC_KEY': public,
    'JWT_PRIVATE_KEY': private,
    'JWT_ALGORITHM': JWT_ALGORITHM
}

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.10/howto/static-files/

STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(BASE_DIR, 'static/')
if not os.path.exists(STATIC_ROOT):
    os.mkdir(STATIC_ROOT)
SCREENSHOT_DIR = os.path.join(STATIC_ROOT, 'screenshots/')
if not os.path.exists(SCREENSHOT_DIR):
    os.mkdir(SCREENSHOT_DIR)

# используется для нотификации sync_proxy
REDIS_TASK_RESULT_CHANNEL = 'TASK_RESULTS_NOTIFY'
# используется для пересылки django сигналов через REDIS
REDIS_DJANGO_SIGNALS = 'DJANGO_SIGNALS'
# используется для передачи запросов от сервера терминации WS к ctrl_msg_worker
REDIS_WS_CTRL_CHANNEL = 'WS_CTRL_CHANNEL'
# префикс используется для передачи сообщений от ctrl_msg_worker к
# серверу терминации WS соединений
REDIS_WS_RESPONSE_CHANNEL_PREFIX = 'WS_RESPONSE'
# метка перезагрузки nginx
REDIS_RELOAD_NGINX = 'RELOAD:NGINX'

# heartbeat settings
ERROR_LABEL_EXPTIME = 30
HERMIT_LABEL_EXPTIME = 60
STORAGE_EXPTIME = 50
GRPC_ERROR_LABEL = '{uuid}-grpc-base-error'
GRPC_HERMIT_LABEL = '{uuid}-grpc-base-hermit'
SECOND_ERROR_LABEL = '{uuid}-second-error'
RESTART_NODE_ENGINE_LABEL = '{uuid}-restart-node-engine'
SECOND_HERMIT_LABEL = '{uuid}-second-hermit'
CHANNEL_CLOSE_REQUESTED_FLAG = "{uuid}-close-request"

# queues
TASK_PRE_PROCESSOR_QUEUE = "TASK_PRE_PROCESSOR_QUEUE"
TASK_PROGRESS_WATCHER_QUEUE = "TASK_PROGRESS_WATCHER_QUEUE"
MULTI_TASK_PROCESSOR_QUEUE = "MULTI_TASK_PROCESSOR_QUEUE"
NODE_MANAGER_QUEUE = 'NODE_MANAGER'
NODE_MANAGER_PROCESS_QUEUE = 'NODE_MANAGER_PROCESS'
NODE_MANAGER_CANCEL_QUEUE = 'NODE_MANAGER_CANCEL_QUEUE'

# sets
EVENT_ERROR_SET = "EVENT_ERROR_SET"
EVENT_FAILED_SET = "EVENT_FAILED_SET"

# storage settings
STORAGE_ROOT_DIR = '/storages'
SHARED_STORAGES_PATHS = [f"{STORAGE_ROOT_DIR}/nfs", f"{STORAGE_ROOT_DIR}/glusterfs", f"{STORAGE_ROOT_DIR}/cifs"]
OCFS2_PATH = f"{STORAGE_ROOT_DIR}/ocfs2"
GFS2_PATH = f"{STORAGE_ROOT_DIR}/gfs2"
GLUSTER_PATH = f"{STORAGE_ROOT_DIR}/gluster"
CLUSTER_STORAGES_PATHS = [OCFS2_PATH, GFS2_PATH, GLUSTER_PATH]
LOCAL_STORAGES_PATH = f"{STORAGE_ROOT_DIR}/local"
ZPOOL_STORAGES_PATH = f"{STORAGE_ROOT_DIR}/zfs"
DEFAULT_DATAPOOL = 'default'

# vdisk settings
VDISK_PART_COPY_VERBOSE_NAME = "{}__PART_COPY"
VDISK_PART_COPY_DESCRIPTION = "This vdisk is a partial copy of {}({}) vdisk"
VDISK_COPY_DESCRIPTION = "This vdisk is copy of {}({}) vdisk"

# network settings
DEFAULT_OVS = 'veil-default'
BLACKHOLE_OVS = 'veil-blackhole'
MGMT_INTERFACE = 'veil-mgmt'
MGMT_INTERFACE_GROUP = 'management'
BLACKHOLE_INTERFACE_GROUP = 'blackhole'
DEFAULT_UPLINK_GROUP = 'default'

# file storage paths
PATH_ISO_STORAGE = '_ISO'
PATH_FILE_STORAGE = '_LIBRARY'
PATH_NODES_HEARTBEAT = '_HEARTBEAT'
PATH_MINIO_STORAGE = '_MINIO'
PATH_TMP = '.TMP'

# name settings
DEFAULT_CONTROLLER = 'Default controller'
DEFAULT_DATACENTER = 'Default location'
DEFAULT_CLUSTER = 'Default cluster'
DEFAULT_DATA_PLANE = 'Default data plane'
DEFAULT_VNET_CPLANE = 'Virtual network policies'
DEFAULT_VNET_POLICY = 'Default virtual networks policy'

# db task cache
DB_TASKS_CACHE_FOLDER = '/var/tmp/db_tasks_cache'
DB_TASK_CHECKER_DELAY_SEC = 300
DB_TASK_CHECKER_CHECK_INTERVAL_SEC = 100

DEFAULT_LOG_ROTATION_PERIOD_DAYS = 6 * 30

# settings for ttl_cache
CACHE_EXPIRE = 5 * 60  # 5 minutes

# Range доступных портов на контроллере
# Используется при выделении порта для открытия nginx stream.
# Максимально возможный порт взят как нижняя граница /proc/sys/net/ipv4/ip_local_port_range
CTRL_AVAILABLE_PORT_RANGE = [5900, 32767]
CTRL_AVAILABLE_PORT_LEN = CTRL_AVAILABLE_PORT_RANGE[1] - CTRL_AVAILABLE_PORT_RANGE[0]
# Путь до директории, в которой хранятся конфиги для удаленного доступа к ВМ
REMOTE_ACCESS_CONF_PATH = '/etc/nginx/streams-enabled'
DOMAIN_REMOTE_ACCESS_TEMPLATE_PATH = os.path.join(BASE_DIR, 'domain/remote_access.conf')
NODE_REMOTE_ACCESS_TEMPLATE_PATH = os.path.join(BASE_DIR, 'node/remote_access.conf')

# upload file settings
UPLOAD_URL_PATTERN = '/upload?token={token}'
DOWNLOAD_URL_PATTERN = '/download?token={token}'
UPLOAD_SETTING_PATTERN = 'upload:{}'
UPLOAD_TOKEN_EXPTIME = 30
UPLOADER_SSH_TIMEOUT = 60
UPLOAD_PORT = 8081
LOGGER_PORT = 8082
API_PORT = 8083

NODE_UPTIME_PREFIX = 'UPTIME_NODE:'
DOMAIN_UPTIME_PREFIX = 'UPTIME_DOMAIN:'
DOMAIN_GA_PREFIX = 'GA_DOMAIN:'
DOMAIN_FAILS_PREFIX = "FAILS_DOMAIN:"
CLUSTER_STATS_CPU_PREFIX = 'STATS_CPU_CLUSTER:'
CLUSTER_STATS_MEM_PREFIX = 'STATS_MEM_CLUSTER:'
NODE_STATS_CPU_PREFIX = 'STATS_CPU_NODE:'
NODE_STATS_MEM_PREFIX = 'STATS_MEM_NODE:'
DOMAIN_STATS_CPU_PREFIX = 'STATS_CPU_DOMAIN:'
DOMAIN_STATS_MEM_PREFIX = 'STATS_MEM_DOMAIN:'

VEIL_PEM = f"{CTRL_PATH}/other/controller.pem"

# Путь к директории, в которой хранятся файлы UEFI
NODE_UEFI_NVRAM_DIR = f"{NODE_PATH}/nvram"

# Путь к UEFI Firmware
NODE_UEFI_FIRMWARE = "/usr/share/OVMF/OVMF_CODE.fd"
NODE_WIN_UEFI_FIRMWARE = "/usr/share/OVMF/OVMF_CODE_4M.ms.fd"

# Prometheus
PROMETHEUS_HOST = 'localhost'
PROMETHEUS_PORT = 9090
PROMETHEUS_NODE_PORT = 9100
PROMETHEUS_LIBVIRT_PORT = 9177

# Loki
LOKI_HOST = 'localhost'
LOKI_PORT = 3100

# Beanstalk
BEANSTALK_HOST = 'localhost'
BEANSTALK_PORT = 11300

# smtp
DEFAULT_SENDER_EMAIL = 'noreply@spacevm.org'

if not os.path.exists(LOGS_DIR):
    os.mkdir(LOGS_DIR)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'syslog': {
            '()': 'veilrpc.utils.logger.TimeFormatter'
        },
        'json': {
            '()': 'veilrpc.utils.logger.JsonFormatter'
        }
    },
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'syslog'
        },
        'django_file': {
            'class': 'logging.FileHandler',
            'filename': os.path.join(LOGS_DIR, 'django.log'),
            'formatter': 'json'
        },
        'controller_file': {
            'class': 'logging.FileHandler',
            'filename': os.path.join(LOGS_DIR, 'controller.log'),
            'formatter': 'json'
        },
    },
    'loggers': {
        'django': {
            'level': 'ERROR',
            'handlers': ['console', 'django_file'],
            'propagate': True,
        },
        'bazalt': {
            'level': 'DEBUG',
            'handlers': ['console', 'controller_file'],
            'propagate': True,
        },
        'bazalt_model': {
            'level': 'DEBUG',
            'handlers': ['console', 'controller_file'],
            'propagate': True,
        },
        'bazalt_event': {
            'level': 'DEBUG',
            'handlers': ['console', 'controller_file'],
            'propagate': True,
        },
        'daphne': {
            'level': 'ERROR',
            'handlers': ['console', 'django_file'],
            'propagate': True,
        },
    }
}

if DEBUG:
    ALLOWED_HOSTS.append('*')
    SWAGGER_SETTINGS = {
        'SECURITY_DEFINITIONS': {
            'api_key': {
                'type': 'apiKey',
                'in': 'header',
                'name': 'Authorization'
            }
        },
        'SECURITY': [{'api_key': []}],
        'USE_SESSION_AUTH': False,
        'JSON_EDITOR': True,
    }

# Изменения значений по умолчанию при переходе к версии 3+
FILE_UPLOAD_PERMISSIONS = None  # в версии 3+ 0o644
X_FRAME_OPTIONS = 'SAMEORIGIN'  # в версии 3+ 'DENY'
SECURE_CONTENT_TYPE_NOSNIFF = False  # в версии 3+ True
DEFAULT_HASHING_ALGORITHM = 'sha1'  # для сохранения совместимости в предыдущими версиями, в новых 'sha256'
SECURE_REFERRER_POLICY = None  # в новых версиях 'same-origin'
DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'
TIMEOUT_DELETE_FILE = 60 * 10  # 10 minutes
TIMEOUT_SHUTDOWN_DOMAIN = 60 * 15  # время ожидания выключения ВМ после которого ВМ выключается принудительно
# pylint: disable=wildcard-import
try:
    from bazalt.settings_local import *  # noqa: F403,F401
except ImportError:
    pass

try:
    from bazalt.settings_system import *  # noqa: F403,F401
except ImportError:
    pass
