#!/bin/sh
color_subr="/usr/share/veil/color.subr"
system_subr="/usr/share/veil/system.subr"
deb_subr="/usr/share/veil/deb.subr"
controller_subr="/usr/share/veil/controller.subr"

set -e
. ${color_subr}
. ${system_subr}
. ${deb_subr}
. ${controller_subr}
set +e

nginx_early()
{
	local _ret=0

	env PAGER=cat ${SYSTEMCTL_CMD} status nginx
	_ret=$?
	[ ${_ret} -eq 0 ] && return 0
	${SYSTEMCTL_CMD} enable nginx
	${SYSTEMCTL_CMD} restart nginx
}

if [ -r /var/lib/ecp-veil/tests/space-controller-first-boot.passed ]; then
	/var/lib/ecp-veil/utils/other/spacevm-maintenance -m off
	err 0 "${N1_COLOR}already first-booted: ${N2_COLOR}/var/lib/ecp-veil/tests/space-controller-first-boot.passed${N0_COLOR}"
fi

if [ ! -r /var/lib/ecp-veil/tests/first_boot.passed ]; then
	maxwait=300
	for i in $( ${SEQ_CMD} 1 ${maxwait} ); do
		if [ ${i} -eq ${maxwait} ]; then
			err 1 "${W1_COLOR}space-utils not ready. ${N1_COLOR}Please checkout /var/log/veil/first_boot.log then re-run: ${N2_COLOR}/var/lib/ecp-veil/controller/other/first_boot.sh${N0_COLOR}"
		fi
		${ECHO} "${N1_COLOR}Wating for space-utils first boot (/var/lib/ecp-veil/tests/first_boot.passed): ${N2_COLOR}${i}/${maxwait}${N0_COLOR}"
		sleep 1
		[ -r /var/lib/ecp-veil/tests/first_boot.passed ] && break
	done
fi


# обслуживаем maintenance. сервис мог быть запущен на стадии space-utils first-boot
nginx_early

# reset progressbar at the end of space-utils frst boot?
space_controller_only=1
[ -r /var/lib/ecp-veil/node/requirements.txt ] && space_controller_only=0

# если нет пакета space-node, то снимаем maintenance в конце скрипта, при этом, процент регулируем до 100 (todo: ввести коэффициент)
# если же пакет space-node существует, процент настройки регулируем лишь до 60%, заглушку не снимаем и передаем регулировку 
# first-boot скрипту пакета node
if [ ${space_controller_only} -eq 1 ]; then
	trap "/var/lib/ecp-veil/utils/other/spacevm-maintenance -m off" HUP INT ABRT BUS TERM EXIT
fi

# space-utils passed
echo "--" >> /var/log/veil/first_boot.log
${ECHO} "${N1_COLOR}space-controller first-boot in progress, logs: ${N2_COLOR}/var/log/veil/first_boot.log${N0_COLOR}" | ${TEE_CMD} -a /var/log/veil/first_boot.log

full_st_time=$( ${DATE_CMD} +%s )

PROGRESS=$( ${AWK_CMD} 'BEGIN{srand();print int(rand()*(20-25))+25 }' )
/var/lib/ecp-veil/utils/other/spacevm-maintenance -m on -p ${PROGRESS} -s "SpaceVM first-boot: configure space-controller: net_helper"

# networking, legacy
${ECHO} "${N1_COLOR}run net_helper...${N0_COLOR}" | ${TEE_CMD} -a /var/log/veil/first_boot.log
/var/lib/ecp-veil/utils/other/net_helper.sh | ${TEE_CMD} -a /var/log/veil/first_boot.log

PROGRESS=$( ${AWK_CMD} 'BEGIN{srand();print int(rand()*(25-30))+30 }' )
/var/lib/ecp-veil/utils/other/spacevm-maintenance -m on -p ${PROGRESS} -s "SpaceVM first-boot: configure space-controller: init_controller_db"


# Controller first boot script wrapper
${ECHO} "${N1_COLOR}space-controller-first-boot exec: ${N2_COLOR}/var/lib/ecp-veil/controller/other/init_controller_db${N0_COLOR}" | ${TEE_CMD} -a /var/log/veil/first_boot.log
env NOCOLOR=1 /var/lib/ecp-veil/controller/other/init_controller_db >> /var/log/veil/first_boot.log 2>&1
_ret=$?

[ ${_ret} -ne 0 ] && err 1 "${W1_COLOR}Controller first boot script failed. ${N1_COLOR}Please checkout /var/log/veil/first_boot.log then re-run: ${N2_COLOR}/var/lib/ecp-veil/controller/other/first_boot.sh${N0_COLOR}"

PROGRESS=$( ${AWK_CMD} 'BEGIN{srand();print int(rand()*(30-40))+40 }' )
/var/lib/ecp-veil/utils/other/spacevm-maintenance -m on -p ${PROGRESS} -s "SpaceVM first-boot: configure space-controller: apply role"


${ECHO} "${N1_COLOR}space-controller-first-boot: exec /usr/local/sbin/veil-role -a profile::custom_facts,redis,space_cli,pgsql,pgbouncer,libvirt,fluentd,space_controller_master${N0_COLOR}" | ${TEE_CMD} -a /var/log/veil/first_boot.log
# todo: alternative .pid file for puppet role to avoid collision with @reboot !
for i in 1 2 3; do
	/usr/local/sbin/veil-role -a profile::custom_facts,redis,space_cli,pgsql,pgbouncer,libvirt,fluentd,space_controller_master | ${TEE_CMD} -a /var/log/veil/first_boot.log
	_ret=$?
	[ ${_ret} -eq 0 ] && break
done

${ECHO} "${N1_COLOR}space-controller-first-boot: exec /root/bin/puppet-apply${N0_COLOR}" | ${TEE_CMD} -a /var/log/veil/first_boot.log
# full pass when no node installation
if [ ! -r /var/lib/ecp-veil/node/manage.py ]; then
	for i in 1 2 3; do
		/root/bin/puppet-apply | ${TEE_CMD} -a /var/log/veil/first_boot.log
		_ret=$?
		[ ${_ret} -eq 0 ] && break
	done
fi

PROGRESS=$( ${AWK_CMD} 'BEGIN{srand();print int(rand()*(40-45))+45 }' )
/var/lib/ecp-veil/utils/other/spacevm-maintenance -m on -p ${PROGRESS} -s "SpaceVM first-boot: configure space-controller: grafana"

# grafana, tech-debt
${ECHO} "${N1_COLOR}space-controller first-boot configure grafana" | ${TEE_CMD} -a /var/log/veil/first_boot.log
configure_grafana | ${TEE_CMD} -a /var/log/veil/first_boot.log

full_end_time=$( ${DATE_CMD} +%s )
full_diff_time=$(( full_end_time - full_st_time ))
# export to metrics
put_prometheus_file_metrics "first_boot" "first_boot_space_controller" ${full_diff_time}
full_diff_time=$( displaytime ${full_diff_time} )

${ECHO} "${N1_COLOR}space-controller first-boot complete successfull in: ${N2_COLOR}${full_diff_time}${N0_COLOR}" | ${TEE_CMD} -a /var/log/veil/first_boot.log


if ${space_controller_only} -eq 1 ]; then
	PROGRESS=$( ${AWK_CMD} 'BEGIN{srand();print int(rand()*(85-99))+99 }' )
else
	PROGRESS=$( ${AWK_CMD} 'BEGIN{srand();print int(rand()*(45-60))+60 }' )
fi

/var/lib/ecp-veil/utils/other/spacevm-maintenance -m on -p ${PROGRESS} -s "SpaceVM first-boot: configure space-controller: start services"

# reload daphna/other
stop_services controller-supervisor gunicorn-proxy gunicorn-uploader gunicorn-ws websockify gunicorn-django daphne-django gunicorn-logger gunicorn-apiclient websockify || true
enable_services controller-supervisor gunicorn-proxy gunicorn-uploader gunicorn-ws websockify gunicorn-django daphne-django gunicorn-logger gunicorn-apiclient websockify || true
start_services controller-supervisor gunicorn-proxy gunicorn-uploader gunicorn-ws websockify gunicorn-django daphne-django gunicorn-logger gunicorn-apiclient websockify


# Get packages stats
calc_deb_packages_status
# ALL_PACKAGES_NUM and ALL_PACKAGES_BYTES
put_prometheus_file_metrics "first_boot" "first_boot_space_controller_all_packages_num" ${ALL_PACKAGES_NUM}
put_prometheus_file_metrics "first_boot" "first_boot_space_controller_all_packages_bytes" ${ALL_PACKAGES_BYTES}

# This file is awaiting a CLI and some postinst scripts of other Space Components
${TOUCH_CMD} /var/lib/ecp-veil/tests/space-controller-first-boot.passed

[ ${space_controller_only} -eq 1 ] && /var/lib/ecp-veil/utils/other/spacevm-maintenance -m off

exit 0
