#!/bin/sh
# вспомогательный хелпер статистики репликации PgSQL, работает из cron.
# TG-22968
system_subr="/usr/share/veil/system.subr"
deb_subr="/usr/share/veil/deb.subr"

EXTRA_MAIN_CMD="psql stat"

set -e
. ${system_subr}
. ${deb_subr}
set +e

#if pg_is_in_recovery; then
#	echo "its slave"
#fi
pg_is_in_recovery()
{
	local _res= _ret=

	_res=$( 2>/dev/null ${SUDO_CMD} -u postgres env PGCONNECT_TIMEOUT=3 ${PSQL_CMD} -tc "select pg_is_in_recovery();" | ${AWK_CMD} '{printf $1}' )
	ret=$?
	[ ${ret} -ne 0 ] && err 1 "unable to get database role via psql: ${_res}"

	case "${_res}" in
		f)
			return 1
			;;
		t)
			return 0
			;;
		*)
			err 1 "error: unknown pg_is_in_recovery status: ${_res}"
			;;
	esac
}

check_slave_config()
{
	local _remote_ip=
	local _ret=
	local _remote_ip_exist=0

	[ ! -r /etc/veil/remote_ip.conf ] && return 0
	_remote_ip=$( ${GREP_CMD} \. /etc/veil/remote_ip.conf | ${AWK_CMD} '{printf $1}' )
	iptype ${_remote_ip}
	_ret=$?

	if [ ${_ret} -ne 1 ]; then
		echo "${0}: check_slave_config: /etc/veil/remote_ip.conf does not contain IPv4: ${_remote_ip}" >> ${DEB_LOGFILE}
		return 0
	fi

	if [ -r /var/lib/ecp-veil/data/local_modules/pgsql.yaml ]; then
		${GREP_CMD} -q ${_remote_ip} /var/lib/ecp-veil/data/local_modules/pgsql.yaml
		_ret=$?
		[ ${_ret} -eq 0 ] && return 0		# slave IP exist
	fi

	# todo: move remote_ip to facts instead of sed
	${SED_CMD} -e "s#%%SLAVE%%#${_remote_ip}#g" \
		/var/lib/ecp-veil/controller/other/pgsql-master.yaml > /var/lib/ecp-veil/data/local_modules/pgsql.yaml

	${CHOWN_CMD} veil-controller:veil /var/lib/ecp-veil/data/local_modules/pgsql.yaml

	/usr/local/sbin/veil-role -a pgsql
	return 0
}

# запускаем analyze
analyze()
{
	local _analyze=0
	local _cur_time= _analyze_mtime= _diff_time

	if [ ! -r ${ANALYZE_STATS} ]; then
		_analyze=1
	else
		_analyze_mtime=$( ${STAT_CMD} -c %Y ${ANALYZE_STATS} )
		_cur_time=$( ${DATE_CMD} +%s )
		_diff_time=$(( _cur_time - _analyze_mtime ))
		[ ${_diff_time} -lt 0 -o ${_diff_time} -gt 86400 ] && _analyze=1
	fi

	[ ${_analyze} -eq 0 ] && return 0

	${ECHO} "${N1_COLOR}VACUUM ANALYZE...${N0_COLOR}"

	2>/dev/null ${SUDO_CMD} -u postgres env PGCONNECT_TIMEOUT=3 ${PSQL_CMD} veildb -tc "VACUUM ANALYZE;" > ${ANALYZE_STATS}
}

# todo: use facts instead of hard-code
PG_VER="13"
PG_DATA_DIR="/var/lib/postgresql/13/main"
DEB_LOGFILE="/var/log/veil/controller/pgstats.log"

PSQL_CMD=$( which psql )
DEST_DIR="/var/cache/spacevm/pgstats"
DEST_STATS="${DEST_DIR}/pgstats.conf"
ANALYZE_STATS="${DEST_DIR}/analyze.conf"

${TRUNCATE_CMD} -s0 ${DEB_LOGFILE}

[ ! -d "${DEST_DIR}" ] && ${MKDIR_CMD} -p ${DEST_DIR}

if pg_is_in_recovery; then
	pg_role="slave"
else
	pg_role="master"
fi

trap "${RM_CMD} -f ${DEST_DIR}/pgstats.conf" HUP INT ABRT BUS TERM EXIT

[ ! -d "${PG_DATA_DIR}" ] && err 1 "no such directory: ${PG_DATA_DIR}"
[ ! -d "${PG_DATA_DIR}/base" ] && err 1 "no such directory: ${PG_DATA_DIR}/base"

# unset trap: all fine
trap "" HUP INT ABRT BUS TERM EXIT

PG_BASE_DIR_BYTES=$( ${DU_CMD} -sb "${PG_DATA_DIR}/base" | ${AWK_CMD} '{printf $1}' )

if [ -d "${PG_DATA_DIR}/pg_wal" ]; then
	PG_WAL_DIR_BYTES=$( ${DU_CMD} -sb "${PG_DATA_DIR}/pg_wal" | ${AWK_CMD} '{printf $1}' )
else
	PG_WAL_DIR_BYTES=0
fi

PG_DATA_DIR_BYTES=$(( PG_BASE_DIR_BYTES + PG_WAL_DIR_BYTES ))

PG_PARTITION_FREE_BYTES=$( ${DF_CMD} -B1 / | ${TAIL_CMD} -n +2 | ${AWK_CMD} '{printf $4}' )

# 2>/dev/null  ${SUDO_CMD} -u postgres env PGCONNECT_TIMEOUT=3 ${PSQL_CMD} -t veildb < /var/lib/ecp-veil/controller/other/pgsql/wasted-bloat.sql > /var/cache/spacevm/pgstats/wasted-bloat.conf			# 0m0.424s
# 2>/dev/null ${SUDO_CMD} -u postgres env PGCONNECT_TIMEOUT=3 ${PSQL_CMD} -t veildb < /var/lib/ecp-veil/controller/other/pgsql/top-tables.sql > /var/cache/spacevm/pgstats/top-tables.conf				# 0m0.424s
pg_database_size_bytes=$( 2>/dev/null ${SUDO_CMD} -u postgres env PGCONNECT_TIMEOUT=3 ${PSQL_CMD} -tc "select pg_database_size('veildb')" | ${AWK_CMD} '{printf $1}' )	# 0m0.333s

case "${pg_role}" in
	slave)
		pg_last_xact_replay=$( 2>>${DEB_LOGFILE} ${SUDO_CMD} -u postgres env PGCONNECT_TIMEOUT=3 ${PSQL_CMD} -tc "SELECT EXTRACT(EPOCH FROM (now() - pg_last_xact_replay_timestamp()))::INT;" | ${AWK_CMD} '{printf $1}' )
		ret=$?
		if [ ${ret} -ne 0 ]; then
			echo "unable to get pg_last_xact_replay psql" >> ${DEB_LOGFILE}
			${CAT_CMD} ${DEB_LOGFILE}
			pg_last_xact_replay="3000"
		fi
		if [ -z "${pg_last_xact_replay}" ]; then
			echo "empty pg_last_xact_replay value, no updates yet?" >> ${DEB_LOGFILE}
			${CAT_CMD} ${DEB_LOGFILE}
			pg_last_xact_replay="3000"
		fi
		;;
	master)
		# update date in sysdev::pgstats table so that the slave can receive pg_last_xact_replay within ~ten minutes
		2>/dev/null ${SUDO_CMD} -u postgres env PGCONNECT_TIMEOUT=3 ${PSQL_CMD} -t sysdev < /var/lib/ecp-veil/controller/other/pgsql/sysdev.sql
		pg_last_xact_replay="0"
		check_slave_config
		analyze
		;;
esac

tmp_file=$( ${MKTEMP_CMD} -p ${DEST_DIR} )
${CAT_CMD} > ${tmp_file} <<EOF
pg_data_dir_bytes="${PG_DATA_DIR_BYTES}"
pg_partition_free_bytes="${PG_PARTITION_FREE_BYTES}"
pg_role="${pg_role}"
pg_database_size_bytes="${pg_database_size_bytes}"
pg_last_xact_replay="${pg_last_xact_replay}"
EOF

old_tmp_file=$( ${READLINK_CMD} ${DEST_STATS} 2>/dev/null )
${LN_CMD} -sf ${tmp_file} ${DEST_STATS}
[ -n "${old_tmp_file}" ] && ${RM_CMD} -f ${old_tmp_file}

exit 0
